/*
 * GStreamer
 * Copyright (C) 2005 Thomas Vander Stichele <thomas@apestaart.org>
 * Copyright (C) 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
 * Copyright (C) 2009 Dmitry Fink <<dmitry.fink@palm.com>>
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Alternatively, the contents of this file may be used under the
 * GNU Lesser General Public License Version 2.1 (the "LGPL"), in
 * which case the following provisions apply instead of the ones
 * mentioned above:
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef __GST_V4L2SRC_H__
#define __GST_V4L2SRC_H__

#include <gst/gst.h>
#include <gst/base/gstpushsrc.h>
#include "gstv4l2buffer.h"
#include "gstsnapshotbuffer.h"

#include<lunaservice.h>
#include <cjson/json.h>


G_BEGIN_DECLS

/* #defines don't like whitespacey bits */
#define GST_TYPE_V4L2SRC \
  (gst_v4l2src_get_type())
#define GST_V4L2SRC(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_V4L2SRC,Gstv4l2src))
#define GST_V4L2SRC_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_V4L2SRC,Gstv4l2srcClass))
#define GST_IS_V4L2SRC(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_V4L2SRC))
#define GST_IS_V4L2SRC_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_V4L2SRC))

#define PREVIEW_FRAMES_NUM  			(4)
#define PREVIEW_PREBUFFER_FRAMES_NUM  	(3)

#define SNAPSHOT_WIDTH			(1600)
#define SNAPSHOT_HEIGHT			(1200)
#define SNAPSHOT_BUFFER_SIZE    (SNAPSHOT_WIDTH * SNAPSHOT_HEIGHT * 3 / 2)

#define VIDEO_WIDTH				(640)
#define VIDEO_HEIGHT			(480)
#define VIDEO_BUFFER_SIZE		(VIDEO_WIDTH * VIDEO_HEIGHT * 3 / 2)

#define THUMBNAIL_WIDTH			(320)
#define THUMBNAIL_HEIGHT		(240)
#define THUMBNAIL_BUFFER_SIZE   (THUMBNAIL_WIDTH * THUMBNAIL_HEIGHT * 3 / 2)


#define PREFLASH_CURRENT	240
#define FLASH_CURRENT		480

#define LOWVOLTAGE_FLASHCURRENT 360

#define PREFLASH_FRAMECOUNT  8
#define AUTO_FRAMECOUNT 5
#define MAX_RETRYCOUNT 200
#define SKIP_FRAMECOUNT 10
#define LUMINANCE_THRESHOLD 0x23


#define URI_TP_SERVICE "palm://com.palm.hidd/HidTouchpanel/State"
#define TP_FLASH_ON    "{\"mode\":\"set\",\"value\":\"flashOn\"}"
#define TP_FLASH_OFF   "{\"mode\":\"set\",\"value\":\"flashOff\"}"

#define BATTERY_VOLTAGE_THRESHOLD 3600

#define JSON_BATTERY_SIGNAL_ADDMATCH "{\"category\":\"/com/palm/power\",\"method\":\"batteryStatus\"}"
#define URI_SIGNAL_ADDMATCH "palm://com.palm.lunabus/signal/addmatch"


#define URI_POWERD_BATTERY_SIGNAL_REQUEST  "palm://com.palm.display/com/palm/power/batteryStatusQuery"
#define JSON_SIGNAL_REQUEST ""


typedef struct _Gstv4l2src			Gstv4l2src;
typedef struct _Gstv4l2srcClass		Gstv4l2srcClass;

struct _Gstv4l2src
{
	GstPushSrc pushsrc;
	
	// This is additinal pad for still pictures
	// Video src pad is created by BaseSrc class
	GstPad *srcpad_still;
	
	// Dimensions of snapshot, preview, thumbnail
	cam_ctrl_dimension_t* dimension;
	
	// Descriptor for v4l2 interface (/dev/video20)
	int fd;
	
	// Preview buffer pool and snapshot buffers
	Gstv4l2buffer* buffers[PREVIEW_FRAMES_NUM];
	GstSnapshotBuffer* snapshot_buffer;

	// Camera state
	int state;

	// Preview frame count
	int frame_count;

	//Pre-flash frame count;
	int preflash_frame_count;

	int flash_mode;
	
	// to turn the flash or not	
	boolean turn_flash;

	int retry_count;
	
	gboolean silent;

	pthread_mutex_t mutex;
	pthread_mutex_t skipframe_mutex;
	int current_state;
	int next_state;
	int skip_framecount;

	LSHandle *pServiceHandle;
	GMainLoop* mainLoop;
	LSError	lserror;

	pthread_mutex_t tp_mutex;
	pthread_cond_t tp_cond;	
	boolean	tp_returnValue;

	int  voltage_level;
	boolean voltage_safe;
	pthread_mutex_t vl_mutex;

	struct timeval tv_start;
	GstClockTime last_video_buffer_ts;
};

struct _Gstv4l2srcClass 
{
	GstPushSrcClass parent_class;
};

GType gst_v4l2src_get_type (void);

G_END_DECLS

#endif /* __GST_V4L2SRC_H__ */
